@@ -17,6 +17,7 @@ module Happstack.Server.Internal.Cookie
     where
 
 import Control.Monad
+import Control.Monad.Fail (MonadFail)
 import qualified Data.ByteString.Char8 as C
 import Data.Char             (chr, toLower)
 import Data.Data             (Data, Typeable)
@@ -24,6 +25,7 @@ import Data.List             ((\\), intersperse)
 import Data.Time.Clock       (UTCTime, addUTCTime, diffUTCTime)
 import Data.Time.Clock.POSIX (posixSecondsToUTCTime)
 import Happstack.Server.Internal.Clock (getApproximateUTCTime)
+import Network.URI           (escapeURIString)
 import Text.ParserCombinators.Parsec hiding (token)
 
 #if MIN_VERSION_time(1,5,0)
@@ -95,21 +97,28 @@ mkCookie key val = Cookie "1" "/" "" key val False False
 -- the first argument to this function.
 mkCookieHeader :: Maybe (Int, UTCTime) -> Cookie -> String
 mkCookieHeader mLife cookie =
-    let l = [("Domain=",  cookieDomain cookie)
-            ,("Max-Age=", maybe "" (show . max 0 . fst) mLife)
-            ,("expires=", maybe "" (formatTime defaultTimeLocale "%a, %d-%b-%Y %X GMT" . snd) mLife)
-            ,("Path=",    cookiePath cookie)
-            ,("Version=", s cookieVersion)]
-        s f | f cookie == "" = ""
-        s f   = '\"' : concatMap e (f cookie) ++ "\""
-        e c | fctl c || c == '"' = ['\\',c]
-            | otherwise          = [c]
-    in concat $ intersperse ";" ((cookieName cookie++"="++s cookieValue):[ (k++v) | (k,v) <- l, "" /= v ] ++
-                                 (if secure cookie then ["Secure"] else []) ++
-                                 (if httpOnly cookie then ["HttpOnly"] else []))
+  let
+    l =
+      [ (,) "Domain="  (cookieDomain cookie)
+      , (,) "Max-Age=" (maybe "" (show . max 0 . fst) mLife)
+      , (,) "expires=" (maybe "" (formatTime'  . snd) mLife)
+      , (,) "Path="    (cookiePath cookie)
+      , (,) "Version=" (s cookieVersion)
+      ]
+    formatTime' =
+      formatTime defaultTimeLocale "%a, %d-%b-%Y %X GMT"
+    encode =
+      escapeURIString
+        (\c -> c `elem` (['A'..'Z'] ++ ['a'..'z'] ++ ['0'..'9'] ++ "-_.~"))
+    s f | f cookie == "" = ""
+        | otherwise      = '\"' : (encode $ f cookie) ++ "\""
+  in
+    concat $ intersperse ";" $
+         (cookieName cookie++"="++s cookieValue):[ (k++v) | (k,v) <- l, "" /= v ]
+      ++ (if secure   cookie then ["Secure"]   else [])
+      ++ (if httpOnly cookie then ["HttpOnly"] else [])
+
 
-fctl :: Char -> Bool
-fctl ch = ch == chr 127 || ch <= chr 31
 
 -- | Not an supported api.  Takes a cookie header and returns
 -- either a String error message or an array of parsed cookies
@@ -169,13 +178,13 @@ cookiesParser = cookies
 -- | Get all cookies from the HTTP request. The cookies are ordered per RFC from
 -- the most specific to the least specific. Multiple cookies with the same
 -- name are allowed to exist.
-getCookies :: Monad m => C.ByteString -> m [Cookie]
+getCookies :: MonadFail m => C.ByteString -> m [Cookie]
 getCookies h = getCookies' h >>=  either (fail. ("Cookie parsing failed!"++)) return
 
 -- | Get the most specific cookie with the given name. Fails if there is no such
 -- cookie or if the browser did not escape cookies in a proper fashion.
 -- Browser support for escaping cookies properly is very diverse.
-getCookie :: Monad m => String -> C.ByteString -> m Cookie
+getCookie :: MonadFail m => String -> C.ByteString -> m Cookie
 getCookie s h = getCookie' s h >>= either (const $ fail ("getCookie: " ++ show s)) return
 
 getCookies' :: Monad m => C.ByteString -> m (Either String [Cookie])
@@ -188,9 +197,8 @@ getCookie' s h = do
     return $ do -- Either
        cooks <- cs
        case filter (\x->(==)  (low s)  (cookieName x) ) cooks of
-            [] -> fail "No cookie found"
+            [] -> Left "No cookie found"
             f -> return $ head f
 
 low :: String -> String
 low = map toLower
-
