@@ -16,6 +16,10 @@ import Control.Monad.Error                       ( ErrorT(ErrorT), runErrorT
                                                  , Error, MonadError, throwError
                                                  , catchError, mapErrorT
                                                  )
+#if MIN_VERSION_base(4,9,0)
+import Control.Monad.Fail                        (MonadFail)
+import qualified Control.Monad.Fail              as Fail
+#endif
 import Control.Monad.Reader                      ( ReaderT(ReaderT), runReaderT
                                                  , MonadReader, ask, local, mapReaderT
                                                  )
@@ -70,7 +74,11 @@ type ServerPart a = ServerPartT IO a
 --
 -- see also: 'simpleHTTP', 'ServerMonad', 'FilterMonad', 'WebMonad', and 'HasRqData'
 newtype ServerPartT m a = ServerPartT { unServerPartT :: ReaderT Request (WebT m) a }
+#if MIN_VERSION_base(4,9,0)
+    deriving (Monad, MonadFail, MonadPlus, Functor)
+#else
     deriving (Monad, MonadPlus, Functor)
+#endif
 
 instance MonadCatch m => MonadCatch (ServerPartT m) where
     catch action handle = ServerPartT $ catch (unServerPartT action) (unServerPartT . handle)
@@ -498,7 +506,12 @@ instance Monad m => Monad (WebT m) where
     {-# INLINE (>>=) #-}
     return a = WebT $ return a
     {-# INLINE return #-}
-    fail s = lift (fail s)
+
+#if MIN_VERSION_base(4,9,0)
+instance MonadFail m => MonadFail (WebT m) where
+#endif
+
+    fail s = lift (Fail.fail s)
 
 -- | 'WebMonad' provides a means to end the current computation
 -- and return a 'Response' immediately.  This provides an
