@@ -1,4 +1,4 @@
-{-# LANGUAGE FlexibleContexts, FlexibleInstances, TypeSynonymInstances, ScopedTypeVariables #-}
+{-# LANGUAGE CPP, FlexibleContexts, FlexibleInstances, TypeSynonymInstances, ScopedTypeVariables #-}
 -- | Functions and classes related to generating a 'Response' and setting the response code. For detailed instruction see the Happstack Crash Course: <http://www.happstack.com/docs/crashcourse/index.html#creating-a-response>
 module Happstack.Server.Response
     ( -- * Converting values to a 'Response'
@@ -97,7 +97,7 @@ class ToMessage a where
     toContentType _ = B.pack "text/plain"
     toMessage :: a -> L.ByteString
     toMessage = error "Happstack.Server.SimpleHTTP.ToMessage.toMessage: Not defined"
-    toResponse:: a -> Response
+    toResponse :: a -> Response
     toResponse val =
         let bs = toMessage val
             res = Response 200 M.empty nullRsFlags bs Nothing
