@@ -7,12 +7,13 @@ module Happstack.Server.Types
      setHeader, setHeaderBS, setHeaderUnsafe,
      addHeader, addHeaderBS, addHeaderUnsafe,
      setRsCode, -- setCookie, setCookies,
-     Conf(..), nullConf, result, resultBS,
+     LogAccess, logMAccess, Conf(..), nullConf, result, resultBS,
      redirect, -- redirect_, redirect', redirect'_,
      isHTTP1_0, isHTTP1_1,
      RsFlags(..), nullRsFlags, contentLength, chunked, noContentLength,
      HttpVersion(..), Length(..), Method(..), Headers, continueHTTP,
-     Host, ContentType(..)
+     Host, ContentType(..),
+     readDec', fromReadS, FromReqURI(..)
     ) where
 
 import Happstack.Server.Internal.Types
